/*
		Gift Catcher
		
	A small Lynx game for the LynXmas 2020 Game Jam :
	https://itch.io/jam/lynxmas2020
	
						by Dr. Ludos (2020)
	
	Get all my other games: 
			http://drludos.itch.io/
	Support my work and get access to betas and prototypes:
			http://www.patreon.com/drludos
*/

#include <lynx.h>
#include <tgi.h>
#include <joystick.h>
#include <stdlib.h>
#include <conio.h>
#include <6502.h>
#include <time.h>
#include <cc65.h>



//========= SPRITES ===========
//The sprite are declared here in the reverse order of the sprite chain (last sprite is on top of this list), so they can reference each other

//Empty sprite data (1px transparent), useful to hide sprite while manipulating their "data" only
extern unsigned char gfx_empty[];

//Score counter (separate digits)
extern unsigned char gfx_0[];
extern unsigned char gfx_1[];
extern unsigned char gfx_2[];
extern unsigned char gfx_3[];
extern unsigned char gfx_4[];
extern unsigned char gfx_5[];
extern unsigned char gfx_6[];
extern unsigned char gfx_7[];
extern unsigned char gfx_8[];
extern unsigned char gfx_9[];
static SCB_RENONE spr_score1D = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	0,
	gfx_0,
	4, 23,
};
static SCB_RENONE spr_score1C = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_score1D,
	gfx_0,
	4, 29,
};
static SCB_RENONE spr_score1B = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_score1C,
	gfx_0,
	4, 35,
};
static SCB_RENONE spr_score1A = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_score1B,
	gfx_0,
	4, 41,
};
//Score label
extern unsigned char gfx_score[];
static SCB_RENONE spr_score1msg = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_score1A,
	gfx_score,
	4, 63,
};

//Foes
extern unsigned char gfx_foe[];
static SCB_RENONE spr_foe7 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_score1msg,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe6 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe7,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe5 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe6,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe4 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe5,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe3 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe4,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe2 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe3,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe1 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe2,
	gfx_foe,
	0, 130,
};
static SCB_RENONE spr_foe0 = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe1,
	gfx_foe,
	0, 130,
};

//Goal
extern unsigned char gfx_gift[];
static SCB_RENONE spr_goal = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_foe0,
	gfx_gift,
	0, 120,
};

//Player (2 animations frames)
extern unsigned char gfx_playerUP[];
extern unsigned char gfx_playerDOWN[];
static SCB_RENONE spr_player = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_goal,
	gfx_playerUP,
	0, 120,
};



//Background
extern unsigned char gfx_bg[];
SCB_REHV_PAL spr_bg = {
	BPP_3 | TYPE_BACKGROUND, //CAREFUL, the BG is in BPP3 mode instead of BPP4 as it contains only 5 differents colors in total (I guess this is SPR65 automatic optimisations...)
	REHV, 
	NO_COLLIDE,
	(char*) &spr_player,
	gfx_bg,
	0, 0,
	0x0100, 0x100,
	{ 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }
};

//The following sprites are not part of the main sprite chain (but used in title screen / game over screens for example)

//Message: "Press Button" 
extern unsigned char gfx_press1[];
SCB_REHV_PAL spr_press1 = {
	BPP_4 | TYPE_NORMAL, 
	REHV, 
	NO_COLLIDE,
	0,
	gfx_press1,
	134, 50,
	0x0100, 0x100,
	{ 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }
};

//Title screen texts sprite chain
//Credits texts
extern unsigned char gfx_drludos[];
extern unsigned char gfx_lynxmas[];
SCB_RENONE spr_creditsB = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL,
	NO_COLLIDE,
	0,
	gfx_lynxmas,
	152, 50,
};
SCB_RENONE spr_creditsA = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL,
	NO_COLLIDE,
	(char*) &spr_creditsB,
	gfx_drludos,
	114, 50,
};
//Title Screen Logo
extern unsigned char gfx_titlelogo[];
SCB_REHV_PAL spr_titlelogo = {
	BPP_4 | TYPE_NORMAL, 
	REHV, 
	NO_COLLIDE,
	(char*) &spr_creditsA,
	gfx_titlelogo,
	30, 11,
	0x0100, 0x100,
	{ 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }
};

//Game Over Logo
extern unsigned char gfx_gameover[];
SCB_REHV_PAL spr_gameover = {
	BPP_4 | TYPE_NORMAL, 
	REHV, 
	NO_COLLIDE,
	0,
	gfx_gameover,
	44, 22,
	0x0100, 0x100,
	{ 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }
};

//New record message
extern unsigned char gfx_record[];
SCB_REHV_PAL spr_newrecord = {
	BPP_4 | TYPE_NORMAL, 
	REHV, 
	NO_COLLIDE,
	0,
	gfx_record,
	24, 50,
	0x0100, 0x100,
	{ 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }
};

//Current highscore message (4 digits, with altered palette, chained to the "best:" label)
extern unsigned char gfx_best[];
static SCB_RENONE spr_highscore1D = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	0,
	gfx_0,
	24, 26,
};
static SCB_RENONE spr_highscore1C = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_highscore1D,
	gfx_0,
	24, 32,
};
static SCB_RENONE spr_highscore1B = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_highscore1C,
	gfx_0,
	24, 38,
};
static SCB_RENONE spr_highscore1A = {
	BPP_4 | TYPE_NORMAL, 
	REUSEPAL, 
	NO_COLLIDE,
	(char*) &spr_highscore1B,
	gfx_0,
	24, 44,
};
//Some colors of this Sprite Control Block palette are "re-arranged" on the fly, and all the other sprite from the chain use it too
//that way, we can reuse the digits from the score counter in a different color)
//(compare this SCB palette to any previous one, you'll see that the colors aren't in the same order)
SCB_REHV_PAL spr_highscore1msg = {
	BPP_4 | TYPE_NORMAL, 
	REHV, 
	NO_COLLIDE,
	(char*) &spr_highscore1A,
	gfx_best,
	24, 63,
	0x0100, 0x100,
	{ 0x01, 0x2D, 0xC5, 0x67, 0x89, 0xAB, 0x43, 0xEF } 
};


//========= VARIABLES ===========

//Input
extern unsigned char checkInput(void);
static unsigned char joy;
extern unsigned char reset; 

//Sound
extern void lynx_snd_init ();
extern void lynx_snd_pause ();
extern void lynx_snd_continue ();
extern void __fastcall__ lynx_snd_play (unsigned char channel, unsigned char *music);
extern void lynx_snd_stop ();

//Get sfx data (must match all the sfx defined in tunes/sfxmusic.s)
typedef struct {
    unsigned char *shoot;
    unsigned char *gotcha;
    unsigned char *ting;
    unsigned char *tingting;
    unsigned char *bonus;
    unsigned char *bad;
    unsigned char *cut;
    unsigned char *bonk;
} sfx_data;
extern sfx_data sfx;

//Get music data (must match all the musics defined in tunes/sfxmusic.s)
typedef struct {
    unsigned char *channel0;
    unsigned char *channel1;
    unsigned char *channel2;
    unsigned char *channel3;
} music_data;
extern music_data music;

//Fade in and Fade out function
extern void __fastcall__ fade_in(char *pal, unsigned char interval);
extern void __fastcall__ fade_out(unsigned char interval);
extern unsigned char mainPal[32];


//Text string array buffer to print variables on screen (20chars*8px=160px)
static char text[20];

//Looping variables
static unsigned char i, j;
static unsigned int k;

//FUNCTIONS
static void initGame();
static void spawnFoe();


//Ticks counter (frame counter + modulo computed each frame to trigger 4 "every X frame" counter, so we can move at speed slower than 60 pixel / seconds - 60fps game baby!)
//Frame counter (resetted every 240 frames (4 seconds), so it can be used for countdown too)
static unsigned char ticks;
//Fast speed : triggered every 2 frames
static unsigned char ticks_fast;
//Mid speed : triggered every 4 frames
static unsigned char ticks_mid;
//Slow speed : triggered every 8 frames
static unsigned char ticks_slow;
//Slower speed : triggered every 16 frames
static unsigned char ticks_slower;

//GAMEPLAY

//General variables
static unsigned char STATE; //Current game state (0: title / 1: gameplay / 2: game over)
static unsigned int gametime; //Countdown used for animation in game over and title screen (in ticks)

//Press the buttons only once at a time
static unsigned char pressedP1;

//Score
static unsigned int P1_score; //Current player score
static unsigned int P1_highscore; //Current player highscore

//Player 1
static unsigned char P1_posX; //X position of the player
static unsigned char P1_posY; //Y position of the player
static signed char P1_speedX; //Current moving speed X of the player (treated like a 4.4 fixed point value in game, but signed, so actually a 3.4 fixed point value)
static signed char P1_speedY; //Current moving speed Y of the player
static unsigned char P1_catch; //Current "catching" state (0: nothing catched (active) / 1: gift catched (active) /  200: falling with nothing catched (dead) / 1: falling gift catched (dead) / 255: ready to be shot (idle) )

//Goal (=> Gifts to catch)
static unsigned char goalX; //X position of the goal
static unsigned char goalY; //Y position of the goal

//Array containing the list of all the "digits" sprites usable on each score display digit
static unsigned char* digits[10];

//Foes (=> flying stuffs to avoid)
typedef struct {
	unsigned char active[8]; //1 if currently active in game
	unsigned char x[8]; //pos X
	unsigned char y[8]; //pos Y
	signed char speedX[8]; //Speed on Y axis
	signed char speedY[8]; //Speed on Y axis
	SCB_RENONE *sprite[8]; //pointer to our SCB tie sprite definition (to allow for easy change of appareance if needed)
} Foe;
//Array of foes ("struct of arrays" instead of "arrays of structs"! Thanks to: https://github.com/ilmenit/CC65-Advanced-Optimizations)
static Foe foes;
//Last activated foe index (they are always activated in the same order in this game)
static unsigned char foeActivateIndex;


//======== MAIN LOOP ===========
void game() {
	
	//-= INIT GAME =-
	
	//Init random generator
	srand(clock());
	
	//Init gameplay variables
	ticks=1;
	P1_score=0;
	P1_highscore=0;
	
	//Set the state as Title Screen
	STATE=0;
	
	//Init the Foes
	for( i = 0 ; i < 8 ; ++i ){
		
		//Disable it
		foes.active[i]=0;
		
		//Set its current SCB
		if( i == 0 ){ foes.sprite[i]=&spr_foe0; }
		else if( i == 1 ){ foes.sprite[i]=&spr_foe1; }
		else if( i == 2 ){ foes.sprite[i]=&spr_foe2; }
		else if( i == 3 ){ foes.sprite[i]=&spr_foe3; }
		else if( i == 4 ){ foes.sprite[i]=&spr_foe4; }
		else if( i == 5 ){ foes.sprite[i]=&spr_foe5; }
		else if( i == 6 ){ foes.sprite[i]=&spr_foe6; }
		else if( i == 7 ){ foes.sprite[i]=&spr_foe7; }
		
		//Hides offscreen
		foes.sprite[i]->hpos=0;
		foes.sprite[i]->vpos=130;
	}
	
	//Hide Player
	spr_player.hpos=0;
	spr_player.vpos=130;
	
	
	//Define the digits for score display
	digits[0]=gfx_0;
	digits[1]=gfx_1;
	digits[2]=gfx_2;
	digits[3]=gfx_3;
	digits[4]=gfx_4;
	digits[5]=gfx_5;
	digits[6]=gfx_6;
	digits[7]=gfx_7;
	digits[8]=gfx_8;
	digits[9]=gfx_9;

	
	//Update screen (so it can appear for the fade in)
	
	//Draw the background
	tgi_sprite(&spr_bg);
	//All the other sprites are also draw automatically, as they are chained to the BG
	
	//Draw Title Screen
	tgi_sprite(&spr_titlelogo);

	//Force a screen render (ask it, then wait for it to be done before continuing the program)
	tgi_updatedisplay();
	while( tgi_busy() );
	
	//Do a fade in
	fade_in(mainPal, 100);
	
	//Never use while(1). Every loop needs to be halted by the reset event (in current case, the module will quit and goes back to intro module as defined by the resident program)
	while (!reset) { 
		
		//Wait for the VBL to have rendered new screen
		//=> tgi_busy == 0 if the VBL interupt have swapped the draw/display screen (double buffering)
		//=> tgi_update puts tgi_busy==1 to marks that the drawscreen/displayscreen "need to be swapped" when the next VBL interupts fire (this interupts will revert tgi_busy back to 0, making it a reliable "waitForNextVBLANK" function)
		if (!tgi_busy()) {
			
			//========================
			//====== GAMEPLAY ========
			//========================
			if( STATE == 1 ){
			
				//=== FRAME & TICKS COUNTER ====
	
				//Increase the frame counter
				++ticks;
				
				//Reset it to zero every 4 seconds (unsigned char > 255 max, so 240 ticks => 4 seconds is ok)
				if (ticks == 241){
					ticks= 1;
				}
				
				//Update the "animation trigger" for everyone
				ticks_fast=!(ticks & 0x1);
				ticks_mid=!(ticks & 0x3);
				ticks_slow=!(ticks & 0x7);
				ticks_slower=!(ticks & 0xF);
				
				
				//-= PLAYER ACTIONS =-
				
				//Retrieve the current keys pressed
				//Instead of calling joy_read() directly, use checkInput() defined in resident.c . 
				//It will return the joy state, but will handle pause and reset events too
				//joy = joy_read(JOY_1);
				joy = checkInput();
	
				
				//-= PLAYER 1 =-
				
				//Press button?
				if( JOY_BTN_FIRE(joy) || JOY_BTN_FIRE2(joy) ){
					
					//Do only once per keypress
					if( pressedP1 == 0 ){
					
						//Shoot down the player if it's ready to shoot
						if( P1_catch == 255 ){
							
							//Launch the player
							//Give it some initial speed
							P1_speedX=24;
							//And teleport it slighty to emphasize the initial movement
							P1_posX += 2;
							
							//Set the player as launched, without any goal catched currently
							P1_catch=0;
							
							//Change player anim
							spr_player.data=gfx_playerDOWN;
							
							//Play SFX
							lynx_snd_play(2, sfx.shoot);
							
							//Do only once per keypress
							pressedP1=1;
						}
						//Else, if we are not dead yet, add a slight "upward" movement to the player
						else if( P1_catch != 200 && P1_catch != 201 ){
							
							//Move it up!
							P1_speedX = -3;
							
							//Change player anim
							spr_player.data=gfx_playerUP;
							
							//DON'T Do only once per keypress (so it repeats when button is hold)
						}
						
						
					}
				}
				//Else, button released
				else {
					pressedP1=0;
					
					//If the player isn't stickied on top of screen and doesn't have a gift
					if( P1_catch == 0 ){
						//Change player anim
						spr_player.data=gfx_playerDOWN;
					}
				}
				
				//Moving Player
				//If the player is currently launched (i.e. moving freely with gravity)
				if( P1_catch != 255 ){
					
					//Oh, gravity! (don't exced the max speed, i.e. 127 as we use a signed 8 bit integer value here, and don't apply it if player is out of screen)
					//REMEMBER: posX is actually the "Y position" as the game is played vertically with the Lynx hold sideways!
					if( P1_speedX < 127 && P1_posX < 170 ){
						//Make player go downwards
						++P1_speedX;
						//Every 2 ticks, go down faster!
						if( ticks_fast ){ ++P1_speedX; }
					}
					
					//Move the player (the speed is set in a 4.4 fixed point format, to better simulate gravity / velocities)
					//So I'm shifting it 4 times to the left to get rid of the "decimal" part, and only use the remaining "integer" (signed, so actually it means -8 to + 7 range, which is enough for our speed needs in a tiny 160*102px screen space)
					P1_posX += (P1_speedX >> 4);
					
					//Are we dead? (having been hit once)
					if( P1_catch == 200 || P1_catch == 201 ){
						
						//If the player goes offscreen
						if( P1_posX >= 170 ){
							
							//Move back to the ground level
							P1_posX=170;
						
							//Stop us so we stay here...
							P1_speedX=0;
							
							//Set the state as Game Over
							STATE=2;
							
							//Activate a "cool down" timer before we can start again
							gametime=120;
							
							//Reset ticks for smooth animation
							ticks=0;
						}
					}
					//Else, we are still alive
					else {
						
						//If the player hits the ground (i.e. rope size limit) while still alive
						if( P1_posX >= 136 ){
							
							//Move back to the ground level
							P1_posX=136;
							
							//Stop us so we stay here...
							P1_speedX=0;
						}
					}
					
					//If the player hits the top
					if( P1_posX <= 6 ){
						
						//Move back to the top
						P1_posX=6;
						
						//Stop us
						P1_speedX=0;
						
						//Change player anim
						spr_player.data=gfx_playerUP;
						
						//Did we catch something?
						if( P1_catch == 1 ){
							
							//Score points!
							++P1_score;
							
							//Update score display
							//We separate each digit in a very dirty way to display it directly using the "digits" sprite list
							k=P1_score;
							spr_score1A.data=digits[k/1000];
							k-=(k/1000)*1000;
							spr_score1B.data=digits[k/100];
							k-=(k/100)*100;
							spr_score1C.data=digits[k/10];
							k-=(k/10)*10;
							spr_score1D.data=digits[k];
							
							//Generate a new goal randomly
							goalX=50+(rand()%90);
							goalY=8+(rand()%86);
							//Update goal sprite positions
							spr_goal.hpos=goalX;
							spr_goal.vpos=goalY;
							
							//Spawn a new foe (actually, we always call the function, and it'll decide whether to spawn foe or not depending on our current score)
							spawnFoe();
							
							//Play SFX
							lynx_snd_play(3, sfx.bonus);
						}
						//Else, we catched nothing
						
						//And set us in "ready to be shot" mode
						P1_catch=255;
						
						//Force a "do only once per keypress", so it doesn't fire again immediatedly
						pressedP1=1;
					}
					
					//GOAL MANAGEMENT
					
					//Did we catch the goal already?
					if( P1_catch == 1 ){

						//Move the goal with the player
						goalX=P1_posX+11;
						goalY=P1_posY;
						
						//Update the sprite position with the new values
						spr_goal.hpos=goalX;
						spr_goal.vpos=goalY;
					}
					//Else, check if we may have just hit the goal! (unless we are falling because the game is over)
					else if ( P1_catch != 200 && P1_catch != 201 ){
						
						//Did we hit the current goal?
						//OPTIMIZATION: Manual "inlining" of collision function to save CPU time
						//if ((x1 < (x2+w2)) && ((x1+w1) > x2) && (y1 < (h2+y2)) && ((y1+h1) > y2))
						if( (P1_posX < (goalX + 15)) && ((P1_posX+22) > goalX) && (P1_posY < (goalY+15)) && ((P1_posY+15) > goalY) ){
							
							//Register the goal as catched!
							P1_catch += 1; //add 1 instead of setting to 1, in case we catch a gift while falling! :)
						
							//Move the goal with the player
							goalX=P1_posX+3;
							goalY=P1_posY;
							
							//Update the sprite position with the new values
							spr_goal.hpos=goalX;
							spr_goal.vpos=goalY;
						
							//Change player anim
							spr_player.data=gfx_playerUP;
						
							//Play SFX
							lynx_snd_play(3, sfx.gotcha);
						}
					}

				}
				//Else, we are waiting to be shot
				else {
					
					//Moves slowly in the Y position only
					P1_posY += P1_speedY;
					
					//Moves back and forth when the movement limits are reached
					//LEFT
					if( P1_posY < 12 ){
						//Move back to the limit
						P1_posY=12;
						//Reverse the Speed
						P1_speedY = -P1_speedY;
					}
					//RIGHT
					else if( P1_posY > 90 ){
						//Move back to the limit
						P1_posY=90;
						//Reverse the Speed
						P1_speedY = -P1_speedY;
					}
				}
				
				//Update the sprite position with the new values
				spr_player.hpos=P1_posX;
				spr_player.vpos=P1_posY;
				
				
				
				//-= FOES =-
				
				//For each FOE
				for( i = 0 ; i < 8 ; ++i ){
					
					//If the foe is active
					if( foes.active[i] == 1 ){
						
						//Move it using its speed variables
						
						//X movement (move every 2 frames only if Y movement is active too)
						if( foes.speedY[i] == 0 || ticks_fast ){
							foes.x[i] += foes.speedX[i];
						}
						
						//Don't go out in X!
						//LEFT
						if( foes.x[i] < 30 ){
							
							//Teleport back to the limit
							foes.x[i]=30;
							
							//Reverse speed
							 foes.speedX[i] = -foes.speedX[i];
						}
						//RIGHT
						else if( foes.x[i] > 146 ){
							
							//Teleport back to the limit
							foes.x[i]=146;
							
							//Reverse speed
							 foes.speedX[i] = -foes.speedX[i];
						}
						
						//Y movement (move every 2 frames only if X movement is active too)
						if( foes.speedX[i] == 0 || ticks_fast ){
							foes.y[i] += foes.speedY[i];
						}
						
						//Don't go out in Y!
						//TOP
						if( foes.y[i] < 6 ){
							
							//Teleport back to the limit
							foes.y[i]=6;
							
							//Reverse speed
							foes.speedY[i] = -foes.speedY[i];
						}
						//BOTTOM
						else if( foes.y[i] > 96 ){
							
							//Teleport back to the limit
							foes.y[i]=96;
							
							//Reverse speed
							foes.speedY[i] = -foes.speedY[i];
						}
						
						//Update sprite positions
						foes.sprite[i]->hpos = foes.x[i];
						foes.sprite[i]->vpos = foes.y[i];
						
						//If the foe hits Player
						//OPTIMIZATION: Manual "inlining" of collision function to save CPU time
						//if ((x1 < (x2+w2)) && ((x1+w1) > x2) && (y1 < (h2+y2)) && ((y1+h1) > y2))
						if( (P1_posX < (foes.x[i]+10)) && ((P1_posX+17) > foes.x[i]) && (P1_posY-3 < (foes.y[i])) && ((P1_posY+3) > foes.y[i]) ){
						
							//If the player is not dead yet
							if( P1_catch != 200 && P1_catch != 201 ){
								
								//Has the player already catched a gift?
								if( P1_catch == 1 ){
									//Then move the Gift offscreen so the player "lose" it while falling
									goalX=170;
									spr_goal.hpos=goalX;
								}
								
								//Disable Player (so it'll fall to the ground!)
								P1_catch = 200; //force us to state 200 instead of 201 if we had a gift, so we always lose our gift!
								
								//Change player anim
								spr_player.data=gfx_playerDOWN;
							}
							
							//Stop player to prepare its downfall (our if we hit player while it's already falling dead)
							P1_speedX=0;
						
							//Make the foe disappear
							foes.active[i]=0;
							foes.x[i] = 0;
							foes.y[i] = 130;
							foes.sprite[i]->hpos = foes.x[i];
							foes.sprite[i]->vpos = foes.y[i];
							
							//Play SFX
							lynx_snd_play(3, sfx.bonk);						
						}
					}
				}
				
				
				
				//-= RENDER SCREEN =-
				
				//Draw the background
				tgi_sprite(&spr_bg);
				//All the other sprites are also draw automatically, as they are chained to the BG
				
				//If the player is not dead yet, draw a "rope" from the top of the screen
				if( P1_catch != 200 && P1_catch != 201 ){
					
					//If the player is below the top GUI (else, we don't draw the line to avoid overlapping the score digits)
					if( P1_posX > 10 ){
						
						//Draw a line for the player from top of screen
						tgi_setcolor(4);
						tgi_line(9, P1_posY, P1_posX-1, P1_posY);
					}
				}
				
			}
			
			//========================
			//===== TITLE SCREEN =======
			//========================
			else if( STATE == 0 ){
				
				//-= PLAYER ACTIONS =-
				
				//Retrieve the current keys pressed
				//Instead of calling joy_read() directly, use checkInput() defined in resident.c . 
				//It will return the joy state, but will handle pause and reset events too
				//joy = joy_read(JOY_1);
				joy = checkInput();
				
				//If any key is pressed
				if( JOY_BTN_FIRE(joy) || JOY_BTN_FIRE2(joy) ){
					
					//Initialize the game
					initGame();
					
					//Mark player buttons as pressed to avoid unintentional action at start
					pressedP1=1;
					
					//Set the gameplay state
					STATE=1;
				}
				
				//-= RENDER SCREEN =-
				
				//BLINKING ANIMATION
				//Increase the frame counter
				++ticks;
				//Reset it to zero every 1 seconds
				if (ticks == 61){	ticks= 1; }
				
				//Draw the background
				tgi_sprite(&spr_bg);
				//All the other sprites are also draw automatically, as they are chained to the BG
				
				//Draw Title Screen
				tgi_sprite(&spr_titlelogo);
				//The other text elements are also drawn automatically, as they are chained to the title screen logo
				
				//Draw the "Press Button" message sprite
				//And makes it blink!
				if( ticks < 31 ){
					tgi_sprite(&spr_press1);
				}
			}
			
			//=======================
			//===== GAME OVER =======
			//=======================
			else if( STATE == 2 ){
				
				//-= PLAYER ACTIONS =-
				
				//Retrieve the current keys pressed
				//Instead of calling joy_read() directly, use checkInput() defined in resident.c . 
				//It will return the joy state, but will handle pause and reset events too
				//joy = joy_read(JOY_1);
				joy = checkInput();
				
				//If any key is pressed
				if( JOY_BTN_FIRE(joy) || JOY_BTN_FIRE2(joy) ){
					
					//Only if the cooldown timer is over
					if( gametime == 0 ){
					
						//If we made a highscore, record it now!
						if( P1_score > P1_highscore ){
							P1_highscore=P1_score;
						}
						
						//Initialize the game
						initGame();
						
						//Mark player buttons as pressed to avoid unintentional action at start
						pressedP1=1;
						
						//Set the gameplay state
						STATE=1;
					}
				}
				
				//Decrease the cooldown timer (to avoid erroneous restart, timer set in the gameplay sequence)
				if( gametime > 0 ){
					--gametime;
				}
				
				
				//-= RENDER SCREEN =-
				
				//BLINKING ANIMATION
				//Increase the frame counter
				++ticks;
				//Reset it to zero every 1 seconds
				if (ticks == 61){	ticks= 1; }
				
				//Draw the background
				tgi_sprite(&spr_bg);
				//All the other sprites are also draw automatically, as they are chained to the BG
				
				//Play SFX (player hitting the ground)
				if( gametime == 110 ){
					lynx_snd_play(3, sfx.bad);
				}
				
				//Wait a few frames before displaying the game over message
				if( gametime <= 60 ){
					
					//Display sprite
					tgi_sprite(&spr_gameover);
				}
				
				//Wait a few frames before displaying the new record / current highscore message
				if( gametime <= 30 ){
					
					//If we made an highscore (we'll register it only when player exits the game over screen, so this condition can be performed each frame while the game over screen is on)
					if( P1_score > P1_highscore ){
						
						//Play victory sound! (do it only once!)
						if( gametime == 30 ){
							//Play SFX
							lynx_snd_play(3, sfx.bonus);
						}
					
						//Display the new record message sprite
						tgi_sprite(&spr_newrecord);
					}
					//Else, we didn't make an highscore, so show the current one
					else {
						//Update highscore display digits (do it only once!)
						if( gametime == 30 ){
							//We separate each digit in a very dirty way to display it directly using the "digits" sprite list
							k=P1_highscore;
							spr_highscore1A.data=digits[k/1000];
							k-=(k/1000)*1000;
							spr_highscore1B.data=digits[k/100];
							k-=(k/100)*100;
							spr_highscore1C.data=digits[k/10];
							k-=(k/10)*10;
							spr_highscore1D.data=digits[k];
						}
						
						//Display the sprite chain with highscore label and digits
						tgi_sprite(&spr_highscore1msg);
					}
				}
				
				
				//Draw the "Press Button" message sprite
				if( gametime == 0 ){
					//And makes it blink!
					if( ticks < 31 ){
						tgi_sprite(&spr_press1);
					}			
				}
			}

			
			//-= COMMON BACKEND =- (for all the states)
			
			//Display new rendered screen by swapping it with the draw buffer (this function just asks for it, the actual swap will be performed by the VBL interupt)
			tgi_updatedisplay();
		}
	}
	
	
	//If the game is resetted (soft reset : Pause + Opt1, have to be handled manually by us!)
	if( reset == 1 ){
		//The main game loop will break automatically, so we do some cleanup / manual reset before exiting this module

		//Disable reset variable (it served its purpose by breaking the game loop already)
		reset=0;
		
		//The fade out and back to menu will be handeld by the main loop in resident.c
	}
}


//======== FUNCTIONS  ===========

//This functions init the game, so it can be replayed instantly without reloading the module :)
static void initGame() {

	//Init gameplay variables
	ticks=1;
	gametime=0; //not used in the game, only in game over screen actually
	
	//Init the Foes
	for( i = 0 ; i < 8 ; ++i ){
		
		//Disable it
		foes.active[i]=0;
		
		//Set its current SCB
		if( i == 0 ){ foes.sprite[i]=&spr_foe0; }
		else if( i == 1 ){ foes.sprite[i]=&spr_foe1; }
		else if( i == 2 ){ foes.sprite[i]=&spr_foe2; }
		else if( i == 3 ){ foes.sprite[i]=&spr_foe3; }
		else if( i == 4 ){ foes.sprite[i]=&spr_foe4; }
		else if( i == 5 ){ foes.sprite[i]=&spr_foe5; }
		else if( i == 6 ){ foes.sprite[i]=&spr_foe6; }
		else if( i == 7 ){ foes.sprite[i]=&spr_foe7; }
		
		//Hides offscreen
		foes.sprite[i]->hpos=0;
		foes.sprite[i]->vpos=130;
	}
	
	//Reset the index used to activate foes sequentially
	foeActivateIndex=0;
	
	//Player 1
	P1_posX=6;
	P1_posY=51;
	P1_speedX=0;
	P1_speedY=1;
	P1_catch=255;
	P1_score=0;
	//Update sprite positions
	spr_player.hpos=P1_posX;
	spr_player.vpos=P1_posY;
	//Reset player anim
	spr_player.data=gfx_playerUP;
	
	//Update score display
	//We separate each digit in a very dirty way to display it directly using the "digits" sprite list
	k=P1_score;
	spr_score1A.data=digits[k/1000];
	k-=(k/1000)*1000;
	spr_score1B.data=digits[k/100];
	k-=(k/100)*100;
	spr_score1C.data=digits[k/10];
	k-=(k/10)*10;
	spr_score1D.data=digits[k];
	
	//Generate a first goal to launch the game
	goalX=140;
	goalY=51;
	//Update sprite positions
	spr_goal.hpos=goalX;
	spr_goal.vpos=goalY;
}


//This functions tries to spawn a new Foe if there are at least one available
static void spawnFoe() {
	
	//Different actions depending on player score
	
	//If we still have foes to spawn
	if( foeActivateIndex < 8 ){
	
		//Spawn a new "horizontal" foe
		if( P1_score == 1 || P1_score == 6 || P1_score == 16  || P1_score == 30 ){
			
			//Activate the foe
			foes.active[foeActivateIndex]=1;
			
			//Position it randomly
			foes.x[foeActivateIndex]=40+(rand()%90);
			foes.y[foeActivateIndex]=8+(rand()%86);
			
			//Move horizontally only (remember, X and Y are reversed as the game is player vertically!)
			foes.speedX[foeActivateIndex]=0;
			foes.speedY[foeActivateIndex]=1;
			
			//Update sprite positions
			foes.sprite[foeActivateIndex]->hpos = foes.x[foeActivateIndex];
			foes.sprite[foeActivateIndex]->vpos = foes.y[foeActivateIndex];
			
			
			//Increase the foe index for the next activation
			++foeActivateIndex;
		}		
		//Else Spawn a new "free movement" foe
		else if( P1_score == 3 || P1_score == 10 || P1_score == 22 || P1_score == 40 ){
			
			//Activate the foe
			foes.active[foeActivateIndex]=1;
			
			//Position it randomly
			foes.x[foeActivateIndex]=40+(rand()%90);
			foes.y[foeActivateIndex]=8+(rand()%86);
			
			//Move both horizontally and vertically (hence, it'll only move once every two frames, see foe update code section)
			foes.speedX[foeActivateIndex]=1;
			foes.speedY[foeActivateIndex]=1;
			
			//Update sprite positions
			foes.sprite[foeActivateIndex]->hpos = foes.x[foeActivateIndex];
			foes.sprite[foeActivateIndex]->vpos = foes.y[foeActivateIndex];
			
			//Increase the foe index for the next activation
			++foeActivateIndex;
		}
		//Else, we don't spawn any foe!
	}
}
